@@ -20,13 +20,14 @@ This package provides functions and classes for lattice modeling.
 from __future__ import absolute_import
 
 import tensorflow_lattice.layers
-import tensorflow_lattice.sonnet_modules
 
 from tensorflow_lattice.python import aggregation_layer
 from tensorflow_lattice.python import categorical_calibration_layer
 from tensorflow_lattice.python import categorical_calibration_lib
 from tensorflow_lattice.python import configs
 from tensorflow_lattice.python import estimators
+from tensorflow_lattice.python import kronecker_factored_lattice_layer
+from tensorflow_lattice.python import kronecker_factored_lattice_lib
 from tensorflow_lattice.python import lattice_layer
 from tensorflow_lattice.python import lattice_lib
 from tensorflow_lattice.python import linear_layer
@@ -36,7 +37,10 @@ from tensorflow_lattice.python import parallel_combination_layer
 from tensorflow_lattice.python import premade
 from tensorflow_lattice.python import premade_lib
 from tensorflow_lattice.python import pwl_calibration_layer
-from tensorflow_lattice.python import pwl_calibration_sonnet_module
 from tensorflow_lattice.python import pwl_calibration_lib
+from tensorflow_lattice.python import pwl_calibration_sonnet_module
 from tensorflow_lattice.python import test_utils
+from tensorflow_lattice.python import utils
 from tensorflow_lattice.python import visualization
+
+import tensorflow_lattice.sonnet_modules
