@@ -17,6 +17,8 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
+import copy
+
 from absl import logging
 from absl.testing import parameterized
 import numpy as np
@@ -210,9 +212,10 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
             pwl_calibration_input_keypoints='uniform',
             pwl_calibration_always_monotonic=False,
             reflects_trust_in=[
-                configs.TrustConfig(feature_name='RM',
-                                    trust_type='edgeworth',
-                                    direction='negative'),
+                configs.TrustConfig(
+                    feature_name='RM',
+                    trust_type='edgeworth',
+                    direction='negative'),
             ],
             regularizer_configs=[
                 configs.RegularizerConfig(name='calib_wrinkle', l2=1e-4),
@@ -245,8 +248,8 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
             name='LSTAT',
             monotonicity=-1,
             dominates=[
-                configs.DominanceConfig(feature_name='AGE',
-                                        dominance_type='monotonic'),
+                configs.DominanceConfig(
+                    feature_name='AGE', dominance_type='monotonic'),
             ],
         ),
     ]
@@ -293,6 +296,10 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
           'age', 'sex', 'cp', 'trestbps', 'chol', 'fbs', 'restecg', 'thalach',
           'exang', 'oldpeak', 'slope', 'ca', 'thal'
       ], 'crystals', 6, 5, True, False, 0.85),
+      ([
+          'age', 'sex', 'cp', 'trestbps', 'chol', 'fbs', 'restecg', 'thalach',
+          'exang', 'oldpeak', 'slope', 'ca', 'thal'
+      ], 'rtl_layer', 6, 5, True, False, 0.85),
   )
   def testCalibratedLatticeEnsembleClassifier(self, feature_names, lattices,
                                               num_lattices, lattice_rank,
@@ -307,6 +314,15 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
         feature_config for feature_config in self.heart_feature_configs
         if feature_config.name in feature_names
     ]
+    if lattices == 'rtl_layer':
+      # RTL Layer only supports monotonicity and bound constraints.
+      feature_configs = copy.deepcopy(feature_configs)
+      for feature_config in feature_configs:
+        feature_config.lattice_size = 2
+        feature_config.unimodality = 'none'
+        feature_config.reflects_trust_in = None
+        feature_config.dominates = None
+        feature_config.regularizer_configs = None
     model_config = configs.CalibratedLatticeEnsembleConfig(
         regularizer_configs=[
             configs.RegularizerConfig(name='torsion', l2=1e-4),
@@ -418,6 +434,10 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
           'CRIM', 'ZN', 'INDUS', 'CHAS', 'NOX', 'RM', 'AGE', 'DIS', 'RAD',
           'TAX', 'PTRATIO', 'B', 'LSTAT'
       ], 'crystals', 6, 5, True, False, 50.0),
+      ([
+          'CRIM', 'ZN', 'INDUS', 'CHAS', 'NOX', 'RM', 'AGE', 'DIS', 'RAD',
+          'TAX', 'PTRATIO', 'B', 'LSTAT'
+      ], 'rtl_layer', 6, 5, True, False, 50.0),
   )
   def testCalibratedLatticeEnsembleRegressor(self, feature_names, lattices,
                                              num_lattices, lattice_rank,
@@ -432,6 +452,15 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
         feature_config for feature_config in self.boston_feature_configs
         if feature_config.name in feature_names
     ]
+    if lattices == 'rtl_layer':
+      # RTL Layer only supports monotonicity and bound constraints.
+      feature_configs = copy.deepcopy(feature_configs)
+      for feature_config in feature_configs:
+        feature_config.lattice_size = 2
+        feature_config.unimodality = 'none'
+        feature_config.reflects_trust_in = None
+        feature_config.dominates = None
+        feature_config.regularizer_configs = None
     model_config = configs.CalibratedLatticeEnsembleConfig(
         regularizer_configs=[
             configs.RegularizerConfig(name='torsion', l2=1e-5),
@@ -565,15 +594,27 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
     self.assertLess(results['average_loss'], average_loss)
 
   @parameterized.parameters(
-      (5, 6, False, True),
-      (4, 5, True, False),
+      ('random', 5, 6, False, True),
+      ('random', 4, 5, True, False),
+      ('rtl_layer', 5, 6, False, True),
+      ('rtl_layer', 4, 5, True, False),
   )
-  def testCalibratedLatticeEnsembleModelInfo(self, num_lattices, lattice_rank,
-                                             separate_calibrators,
+  def testCalibratedLatticeEnsembleModelInfo(self, lattices, num_lattices,
+                                             lattice_rank, separate_calibrators,
                                              output_calibration):
     self._ResetAllBackends()
+    feature_configs = copy.deepcopy(self.heart_feature_configs)
+    if lattices == 'rtl_layer':
+      # RTL Layer only supports monotonicity and bound constraints.
+      for feature_config in feature_configs:
+        feature_config.lattice_size = 2
+        feature_config.unimodality = 'none'
+        feature_config.reflects_trust_in = None
+        feature_config.dominates = None
+        feature_config.regularizer_configs = None
     model_config = configs.CalibratedLatticeEnsembleConfig(
-        feature_configs=self.heart_feature_configs,
+        feature_configs=feature_configs,
+        lattices=lattices,
         num_lattices=num_lattices,
         lattice_rank=lattice_rank,
         separate_calibrators=separate_calibrators,
@@ -611,19 +652,16 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
     self.assertLen(model.nodes, expected_num_nodes)
 
   @parameterized.parameters(
-      (['ZN', 'INDUS', 'RM'], 'random', 3, 1,
-       [['ZN', 'RM'], ['RM'], ['INDUS']]),
-      (['ZN', 'INDUS', 'RM'], 'crystals', 3, 1,
-       [['RM'], ['INDUS'], ['ZN', 'RM']]),
-      (['RM', 'LSTAT', 'AGE'], 'crystals', 3, 1,
-       [['LSTAT'], ['LSTAT', 'AGE'], ['RM']]),
+      (['ZN', 'INDUS', 'RM'], 'random', 3, 1, [['ZN', 'RM'], ['RM'], ['INDUS']
+                                              ]),
+      (['ZN', 'INDUS', 'RM'], 'crystals', 3, 1, [['RM'], ['INDUS'],
+                                                 ['ZN', 'RM']]),
+      (['RM', 'LSTAT', 'AGE'], 'crystals', 3, 1, [['LSTAT'], ['LSTAT', 'AGE'],
+                                                  ['RM']]),
   )
-  def testCalibratedLatticeEnsembleFix2dConstraintViolations(self,
-                                                             feature_names,
-                                                             lattices,
-                                                             num_lattices,
-                                                             lattice_rank,
-                                                             expected_lattices):
+  def testCalibratedLatticeEnsembleFix2dConstraintViolations(
+      self, feature_names, lattices, num_lattices, lattice_rank,
+      expected_lattices):
     self._ResetAllBackends()
     feature_columns = [
         feature_column for feature_column in self.boston_feature_columns
@@ -652,8 +690,7 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
     # Serving input fn is used to create saved models.
     serving_input_fn = (
         tf.estimator.export.build_parsing_serving_input_receiver_fn(
-            feature_spec=fc.make_parse_example_spec(feature_columns))
-    )
+            feature_spec=fc.make_parse_example_spec(feature_columns)))
     saved_model_path = estimator.export_saved_model(estimator.model_dir,
                                                     serving_input_fn)
     logging.info('Model exported to %s', saved_model_path)
@@ -661,8 +698,8 @@ class CannedEstimatorsTest(parameterized.TestCase, tf.test.TestCase):
     lattices = []
     for node in model.nodes:
       if isinstance(node, model_info.LatticeNode):
-        lattices.append([input_node.input_node.name
-                         for input_node in node.input_nodes])
+        lattices.append(
+            [input_node.input_node.name for input_node in node.input_nodes])
 
     self.assertLen(lattices, len(expected_lattices))
     for lattice, expected_lattice in zip(lattices, expected_lattices):
